#!/usr/bin/python3

import pytest
import yaml
import pkg_resources
from influxdb import InfluxDBClient
from clmctest.monitoring.StreamingSim import Sim


@pytest.fixture(scope="module")
def streaming_sim_config():
    """
    Reads the service configuration deployed for the streaming simulation test.

    :return: the python object representing the read YAML file
    """
    rspec = pkg_resources.resource_filename('clmctest.monitoring', 'rspec.yml')
    print("\nrspec file: {0}".format(rspec))

    with open(rspec, 'r') as stream:
        data_loaded = yaml.load(stream)
    return data_loaded


@pytest.fixture(params=[{'database': 'CLMCMetrics'}], scope='module')
def influx_db(streaming_sim_config, request):
    """
    Creates an Influx DB client for the CLMC metrics database

    :param streaming_sim_config: the fixture returning the yaml configuration
    :param request: access the parameters of the fixture
    :return: the created Influx DB client
    """

    return InfluxDBClient(host=streaming_sim_config['hosts'][0]['ip_address'], port='8086', database=request.param['database'], timeout=10)


@pytest.fixture(scope="module")
def simulator(streaming_sim_config):

    influx_url     = "http://" + streaming_sim_config['hosts'][0]['ip_address'] + ":8086"
    influx_db_name = streaming_sim_config['hosts'][1]['database_name']
    agent1_url     = "http://" + streaming_sim_config['hosts'][1]['ip_address'] + ":8186"
    agent2_url     = "http://" + streaming_sim_config['hosts'][2]['ip_address'] + ":8186"

    simulator = Sim(influx_url, influx_db_name, agent1_url, agent2_url)

    simulator.reset()

    return simulator
